import unittest,time

import logging
from ibm_watson_machine_learning.tests.Cloud.preparation_and_cleaning import *

class TestDataAssets(unittest.TestCase):

    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestDataAssets.logger.info("Service Instance: setting up credentials")

        self.wml_credentials = get_wml_credentials()
        self.client = get_client()

        self.cos_credentials = get_cos_credentials()
        self.cos_resource_crn = self.cos_credentials['resource_instance_id']
        self.space_name = str(uuid.uuid4())

        metadata = {
                     self.client.spaces.ConfigurationMetaNames.NAME: 'space' + self.space_name,
                     self.client.spaces.ConfigurationMetaNames.DESCRIPTION: self.space_name + ' description',
                     self.client.spaces.ConfigurationMetaNames.STORAGE: {
                                                                           "type": "bmcos_object_storage",
                                                                           "resource_crn": self.cos_resource_crn
                                                                        }
        }

        self.space = self.client.spaces.store(meta_props=metadata, background_mode=False)

        TestDataAssets.space_id = self.client.spaces.get_id(self.space)
        # TestRshinyApp.space_id = '9cf73498-72d6-42eb-a6f4-424616de9f45'
        print('space_id: ', TestDataAssets.space_id)
        self.client.set.default_space(TestDataAssets.space_id)

    def test_01_create_data_asset(self):
        asset_meta_props = {
            self.client.data_assets.ConfigurationMetaNames.NAME: "test data asset",
            self.client.data_assets.ConfigurationMetaNames.DESCRIPTION: "test data asset",
            self.client.data_assets.ConfigurationMetaNames.DATA_CONTENT_NAME: "artifacts/GoSales_Tx_NaiveBayes.csv"
        }
        asset_details = self.client.data_assets.store(asset_meta_props)
        print(asset_details)

        TestDataAssets.asset_id = self.client.data_assets.get_id(asset_details)
        self.assertIsNotNone(TestDataAssets.asset_id)

    def test_02_get_details(self):

        details = self.client.data_assets.get_details(TestDataAssets.asset_id)
        print(details)
        self.assertTrue(TestDataAssets.asset_id in str(details))

    def test_03_list(self):
        self.client.data_assets.list()

    def test_04_download(self):
        try:
            os.remove('test.csv')
        except:
            pass
        self.client.data_assets.download(TestDataAssets.asset_id, filename='test.csv')
        try:
            os.remove('test.csv')
        except:
            pass

    def test_05_delete_space(self):
        self.client.data_assets.delete(TestDataAssets.asset_id)
        self.client.spaces.delete(TestDataAssets.space_id)

if __name__ == '__main__':
    unittest.main()




import abc
import os

import unittest
from datetime import datetime

from sklearn.pipeline import Pipeline

from ibm_watson_machine_learning import APIClient
from ibm_watson_machine_learning.experiment import AutoAI
from ibm_watson_machine_learning.preprocessing import DataJoinGraph

from ibm_watson_machine_learning.deployment import WebService, Batch
from ibm_watson_machine_learning.workspace import WorkSpace
from ibm_watson_machine_learning.helpers.connections import DataConnection, DeploymentOutputAssetLocation, S3Location
from ibm_watson_machine_learning.experiment.autoai.optimizers import RemoteAutoPipelines
from ibm_watson_machine_learning.tests.utils import (get_wml_credentials, get_cos_credentials, get_space_id,
                                                     save_multiple_data_to_cos_bucket)
from ibm_watson_machine_learning.tests.utils.cleanup import space_cleanup, delete_model_deployment
from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, RunStateTypes

from ibm_watson_machine_learning.tests.utils.assertions import get_and_predict_all_pipelines_as_lale, \
    validate_autoai_experiment

from ibm_watson_machine_learning.preprocessing.data_join_pipeline import OBMPipelineGraphBuilder

get_step_details = OBMPipelineGraphBuilder.get_step_details
get_join_extractors = OBMPipelineGraphBuilder.get_join_extractors
get_extractor_columns = OBMPipelineGraphBuilder.get_extractor_columns
get_extractor_transformations = OBMPipelineGraphBuilder.get_extractor_transformations


class AbstractTestOBM(abc.ABC):
    """
    The abstract tests which covers:
    - training OBM+AutoAI model on a dataset
    - downloading all generated pipelines to lale pipeline
    In order to execute test connection definitions must be provided
    in inheriting classes.
    """

    bucket_name = os.environ.get('BUCKET_NAME', "wml-autoaitests-qa")
    pod_version = os.environ.get('KB_VERSION', None)
    space_name = os.environ.get('SPACE_NAME', 'regression_tests_sdk_space')

    cos_endpoint = "https://s3.us.cloud-object-storage.appdomain.cloud"
    results_cos_path = 'results_wml_autoai'

    # to be set in every child class:
    OPTIMIZER_NAME = "AutoAI + OBM regression test"

    SPACE_ONLY = True
    HISTORICAL_RUNS_CHECK = True

    OBM = True

    BATCH_DEPLOYMENT_WITH_DA = True  # batch input passed as DataConnection type data-assets with csv files
    BATCH_DEPLOYMENT_WITH_CDA = True  # batch input passed as DataConnection type data-assets with connection_id(COS)
    BATCH_DEPLOYMENT_WITH_CA = True  # batch input passed as DataConnection type connected assets (COS)

    DEPLOYMENT_NAME = "SDK tests Deployment"

    experiment_info = dict(name=OPTIMIZER_NAME,
                           desc='test description',
                           prediction_type=PredictionType.MULTICLASS,
                           prediction_column='species',
                           autoai_pod_version=pod_version
                           )

    input_data_filenames: list = None
    input_data_path: str = None

    pipeline_name_to_deploy = "Pipeline_1"

    wml_client: 'APIClient' = None
    data_join_graph: 'DataJoinGraph' = None
    experiment: 'AutoAI' = None
    remote_auto_pipelines: 'RemoteAutoPipelines' = None
    wml_credentials = None
    cos_credentials = None
    pipeline_opt: 'RemoteAutoPipelines' = None

    service_batch: 'Batch'

    batch_payload_location: str
    batch_output_filename = f"batch_output_{datetime.utcnow().isoformat()}.csv"

    cos_resource_instance_id = None
    experiment_info: dict = None

    data_connections: list = None

    trained_pipeline_details = None
    run_id = None
    prev_run_id = None
    results_connection = None
    train_data = None

    pipeline: 'Pipeline' = None
    lale_pipeline = None
    deployed_pipeline = None
    hyperopt_pipelines = None
    new_pipeline = None
    new_sklearn_pipeline = None
    X_df = None
    X_values = None
    y_values = None

    project_id = None
    space_id = None

    asset_id = None
    connection_id = None

    data_assets_to_delete: list = []

    @classmethod
    def setUpClass(cls) -> None:
        """
        Load WML credentials from config.ini file based on ENV variable.
        """
        cls.wml_credentials = get_wml_credentials()
        cls.wml_client = APIClient(wml_credentials=cls.wml_credentials.copy())

        cls.cos_credentials = get_cos_credentials()
        cls.cos_endpoint = cls.cos_credentials.get('endpoint_url')
        cls.cos_resource_instance_id = cls.cos_credentials.get('resource_instance_id')

        cls.project_id = cls.wml_credentials.get('project_id')

    def test_00a_space_cleanup(self):
        space_checked = False
        while not space_checked:
            space_cleanup(self.wml_client,
                          get_space_id(self.wml_client, self.space_name,
                                       cos_resource_instance_id=self.cos_resource_instance_id),
                          days_old=7)
            space_id = get_space_id(self.wml_client, self.space_name,
                                    cos_resource_instance_id=self.cos_resource_instance_id)
            try:
                self.assertIsNotNone(space_id, msg="space_id is None")
                space_checked = True
            except AssertionError:
                space_checked = False

        AbstractTestOBM.space_id = space_id

        if self.SPACE_ONLY:
            self.wml_client.set.default_space(self.space_id)
        else:
            self.wml_client.set.default_project(self.project_id)

    @abc.abstractmethod
    def test_01_create_multiple_data_connections__connections_created(self):
        # List of data connection: `data_connections`
        pass

    @abc.abstractmethod
    def test_02_create_data_join_graph__graph_created(self):
        pass
        # data_join_graph = DataJoinGraph()
        #
        # for f_name in self.input_data_filenames:
        #     data_join_graph.node(name=f_name.split('.')[0])
        # data_join_graph.edge(from_node="main", to_node="customers",
        #                      from_column=["group_customer_id"], to_column=["group_customer_id"])
        # data_join_graph.edge(from_node="main", to_node="transactions",
        #                      from_column=["transaction_id"], to_column=["transaction_id"])
        # data_join_graph.edge(from_node="main", to_node="purchase",
        #                      from_column=["group_id"], to_column=["group_id"])
        # data_join_graph.edge(from_node="transactions", to_node="products",
        #                      from_column=["product_id"], to_column=["product_id"])
        #
        # AbstractTestOBM.data_join_graph = data_join_graph

    def test_04_initialize_AutoAI_experiment__pass_credentials__object_initialized(self):
        if self.SPACE_ONLY:
            AbstractTestOBM.experiment = AutoAI(wml_credentials=self.wml_credentials.copy(),
                                                space_id=self.space_id)
        else:
            AbstractTestOBM.experiment = AutoAI(wml_credentials=self.wml_credentials.copy(),
                                                project_id=self.project_id)

        self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")

    def test_05_initialize_optimizer(self):
        # check if data_join_graph is in experiment info

        AbstractTestOBM.experiment_info = validate_autoai_experiment(self.experiment_info,
                                                                     self.pod_version,
                                                                     self.data_join_graph)

        self.assertIsNotNone(AbstractTestOBM.experiment_info.get("data_join_graph"))

        AbstractTestOBM.remote_auto_pipelines = self.experiment.optimizer(**AbstractTestOBM.experiment_info)

        self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
                              msg="experiment.optimizer did not return RemoteAutoPipelines object")

    def test_06_get_configuration_parameters_of_remote_auto_pipeline(self):
        print("Getting experiment configuration parameters...")
        parameters = self.remote_auto_pipelines.get_params()
        print(parameters)
        self.assertIsInstance(parameters, dict, msg='Config parameters are not a dictionary instance.')

    def test_08_fit_run_training_of_auto_ai_in_wml(self):
        print("Scheduling OBM + KB training...")

        AbstractTestOBM.trained_pipeline_details = self.remote_auto_pipelines.fit(
            training_data_reference=self.data_connections,
            training_results_reference=self.results_connection,
            background_mode=False)

        AbstractTestOBM.run_id = self.trained_pipeline_details['metadata']['id']

        status = self.remote_auto_pipelines.get_run_status()
        run_details = self.remote_auto_pipelines.get_run_details().get('entity')
        self.assertNotIn(status, ['failed', 'canceled'], msg=f"Training finished with status {status}. \n"
                                                             f"Details: {run_details.get('status')}")

        # TestOBMAndKB.model_location = \
        #     self.trained_pipeline_details['entity']['status']['metrics'][-1]['context']['intermediate_model'][
        #         'location'][
        #         'model']
        # TestOBMAndKB.training_status = self.trained_pipeline_details['entity']['results_reference']['location'][
        #     'training_status']

        for connection in self.data_connections:
            self.assertIsNotNone(
                connection.auto_pipeline_params,
                msg=f'DataConnection auto_pipeline_params was not updated for connection: {connection.id}')

    def test_09_download_original_training_data(self):
        print("Downloading each training file...")
        for connection in self.remote_auto_pipelines.get_data_connections():
            train_data = connection.read()

            print(f"Connection: {connection.id} - train data sample:")
            print(train_data.head())
            self.assertGreater(len(train_data), 0)

    def test_10_download_preprocessed_obm_training_data(self):
        print("Downloading OBM preprocessed training data with holdout split...")
        AbstractTestOBM.train_data = (
            self.remote_auto_pipelines.get_preprocessed_data_connection().read())

        AbstractTestOBM.X_df = self.train_data.drop([self.experiment_info['prediction_column']], axis=1)[
                               :10]
        AbstractTestOBM.X_values = AbstractTestOBM.X_df.values
        AbstractTestOBM.y_values = self.train_data[self.experiment_info['prediction_column']][:10]

        print("OBM train data sample:")
        print(self.train_data.head())
        self.assertGreater(len(self.train_data), 0)

    def test_11_get_run_details(self):
        parameters = self.remote_auto_pipelines.get_run_details()
        import json
        print(json.dumps(self.wml_client.training.get_details(training_uid=parameters['metadata']['id']), indent=4))
        print(parameters)
        self.assertIsNotNone(parameters)

    def test_12_get_metrics(self):
        metrics = self.wml_client.training.get_metrics(self.run_id)
        self.assertIsNotNone(metrics)

    def test_13_visualize_obm_pipeline(self):
        print("Visualizing OBM model ...")
        AbstractTestOBM.data_join_pipeline = self.remote_auto_pipelines.get_preprocessing_pipeline()
        assert isinstance(self.data_join_pipeline._pipeline_json, list)

    def test_14_check_if_data_join_pipeline_graph_correct(self):
        pipeline_json = self.data_join_pipeline._pipeline_json
        graph = self.data_join_pipeline._graph_json

        step_types = [message['feature_engineering_components']['obm'][0]['step_type'] for message in pipeline_json]
        last_non_join_iteration = step_types.index('join')
        selection_iteration = step_types.index('feature selection') + 1
        join_iterations = [i + 1 for i, x in enumerate(step_types) if x == "join"]

        for message in pipeline_json:
            name, iteration, _ = get_step_details(message)
            self.assertTrue(str(iteration) in graph['nodes'])

            if 1 < iteration <= 2:
                self.assertTrue(str(iteration) in graph['edges'][str(iteration - 1)])
            elif iteration in join_iterations:
                self.assertTrue(str(iteration) in graph['edges'][str(last_non_join_iteration)])
                extractors = get_join_extractors(message)

                if extractors is None:
                    continue
                for ext, i in zip(extractors, range(len(extractors))):
                    ext_index = str(iteration) + str(i)
                    self.assertTrue(ext_index in graph['nodes'] and
                                    ext_index in ext_index in graph['edges'][str(iteration)])

                    columns = get_extractor_columns(extractors[ext])
                    transformations = get_extractor_transformations(extractors[ext])
                    for j, column in enumerate(columns):
                        col_index = str(iteration) + str(i) + str(j)
                        self.assertTrue(col_index in graph['nodes'] and col_index in graph['edges'][str(ext_index)])

                        for transformation in transformations:
                            self.assertTrue(transformation in graph['edges'][str(col_index)])
                            self.assertTrue(str(selection_iteration) in graph['edges'][str(transformation)])

            elif iteration > selection_iteration:
                self.assertTrue(str(iteration) in graph['edges'][str(iteration - 1)])

    def test_14_predict_using_fitted_pipeline(self):
        print("Make predictions on best pipeline...")
        predictions = self.remote_auto_pipelines.predict(
            X=self.X_values)
        print(predictions)
        self.assertGreater(len(predictions), 0)

    def test_15_summary_listing_all_pipelines_from_wml(self):
        pipelines_summary = self.remote_auto_pipelines.summary()
        print(pipelines_summary.to_string())

        self.assertGreater(len(pipelines_summary), 0, msg=f"Summary is empty:\n {pipelines_summary.to_string()}")
        if 'max_number_of_estimators' in self.experiment_info and self.experiment_info:
            nb_gen_pipelines = 4 * self.experiment_info['max_number_of_estimators']
            self.assertEqual(len(pipelines_summary), nb_gen_pipelines,
                             msg=f"Incorrect pipelines in summary:\n {pipelines_summary.to_string()}")

    def test_16__get_data_connections__return_a_list_with_data_connections_with_optimizer_params(self):
        print("Getting all data connections...")
        data_connections = self.remote_auto_pipelines.get_data_connections()
        self.assertIsInstance(data_connections, list, msg="There should be a list container returned")
        self.assertIsInstance(data_connections[0], DataConnection,
                              msg="There should be a DataConnection object returned")

    def test_17_get_pipeline_params_specific_pipeline_parameters(self):
        print("Getting details of Pipeline_1...")
        pipeline_params = self.remote_auto_pipelines.get_pipeline_details(pipeline_name='Pipeline_1')
        print(pipeline_params)

    def test_18__get_pipeline_params__fetch_best_pipeline_parameters__parameters_fetched_as_dict(self):
        print("Getting details of the best pipeline...")
        best_pipeline_params = self.remote_auto_pipelines.get_pipeline_details()
        print(best_pipeline_params)

    ########
    # LALE #
    ########

    def test_13_get_pipeline__load_lale_pipeline__pipeline_loaded(self):
        AbstractTestOBM.lale_pipeline = self.remote_auto_pipelines.get_pipeline()
        print(f"Fetched pipeline type: {type(self.lale_pipeline)}")
        self.assertWarnsRegex(Warning, "OBM",
                              msg="Warning (OBM pipeline support only for inspection and deployment) was not raised")
        print(f"Fetched pipeline type: {type(self.lale_pipeline)}")

        from lale.operators import TrainablePipeline
        self.assertIsInstance(self.lale_pipeline, TrainablePipeline)

    def test_14_get_all_pipelines_as_lale(self):
        get_and_predict_all_pipelines_as_lale(self.remote_auto_pipelines, self.X_values)

    def test_20__pretty_print_lale__checks_if_generated_python_pipeline_code_is_correct(self):
        pipeline_code = self.lale_pipeline.pretty_print()
        try:
            exec(pipeline_code)

        except Exception as exception:
            self.assertIsNone(
                exception,
                msg=f"Pretty print from lale pipeline was not successful \n\n Full pipeline code:\n {pipeline_code}")

    #################################
    #        HISTORICAL RUNS        #
    #################################

    def test_21_list_historical_runs_and_get_run_ids(self):
        if not self.HISTORICAL_RUNS_CHECK:
            self.skipTest("Skipping historical runs check.")
        runs_df = self.experiment.runs(filter=self.OPTIMIZER_NAME).list()
        print(runs_df)
        self.assertIsNotNone(runs_df)
        self.assertGreater(len(runs_df), 0)

        runs_completed_df = runs_df[runs_df.state == 'completed']

        if len(runs_completed_df) > 1:
            AbstractTestOBM.prev_run_id = runs_completed_df.run_id.iloc[1]  # prev run_id
            print("Random historical run_id: {}".format(AbstractTestOBM.prev_run_id))
            self.assertIsNotNone(AbstractTestOBM.prev_run_id)

    def test_22_get_params_of_last_historical_run(self):
        if not self.HISTORICAL_RUNS_CHECK:
            self.skipTest("Skipping historical runs check.")

        run_params = self.experiment.runs.get_params(run_id=self.run_id)
        self.assertIn('prediction_type', run_params,
                      msg="prediction_type field not fount in run_params. Run_params are: {}".format(run_params))

        AbstractTestOBM.historical_opt = self.experiment.runs.get_optimizer(self.run_id)
        self.assertIsInstance(self.historical_opt, RemoteAutoPipelines,
                              msg="historical_optimizer is not type RemoteAutoPipelines. It's type of {}".format(
                                  type(self.historical_opt)))

        train_data = self.historical_opt.get_data_connections()[0].read()

    def test_23_get_last_historical_pipeline_and_predict_on_historical_pipeline(self):
        if not self.HISTORICAL_RUNS_CHECK:
            self.skipTest("Skipping historical runs check.")

        print("Getting pipeline for last run_id={}".format(self.run_id))
        summary = self.historical_opt.summary()
        pipeline_name = summary.index.values[0]
        historical_pipeline = self.historical_opt.get_pipeline(pipeline_name,
                                                               astype=self.experiment.PipelineTypes.SKLEARN)
        print(type(historical_pipeline))
        predictions = historical_pipeline.predict(self.X_values)
        print(predictions)
        self.assertGreater(len(predictions), 0, msg="Empty predictions")

    ########################################################
    #  Batch deployment (possible test numbers are: 41-54) #
    ########################################################

    def test_40_deployment_target_space_setup(self):
        # note: if target_space_id is not set, use the space_id
        if self.target_space_id is None:
            AbstractTestOBM.target_space_id = self.space_id
        else:
            AbstractTestOBM.target_space_id = self.target_space_id
        # end note

        self.wml_client.set.default_space(AbstractTestOBM.target_space_id)

    def test_41_batch_deployment_setup_and_preparation(self):

        self.assertIsNotNone(AbstractTestOBM.target_space_id, "Test issue: target space not set.")

        if self.SPACE_ONLY:
            AbstractTestOBM.service_batch = Batch(source_wml_credentials=self.wml_credentials,
                                                  source_space_id=self.space_id,
                                                  target_wml_credentials=self.wml_credentials,
                                                  target_space_id=AbstractTestOBM.target_space_id)
        else:
            AbstractTestOBM.service_batch = Batch(source_wml_credentials=self.wml_credentials,
                                                  source_project_id=self.project_id,
                                                  target_wml_credentials=self.wml_credentials,
                                                  target_space_id=AbstractTestOBM.target_space_id)

        self.assertIsInstance(AbstractTestOBM.service_batch, Batch, msg="Deployment is not of Batch type.")
        self.assertIsInstance(AbstractTestOBM.service_batch._source_workspace, WorkSpace,
                              msg="Workspace set incorrectly.")
        self.assertEqual(AbstractTestOBM.service_batch.id, None, msg="Deployment ID initialized incorrectly")
        self.assertEqual(AbstractTestOBM.service_batch.name, None, msg="Deployment name initialized incorrectly")

    def test_42_deploy__batch_deploy_best_computed_pipeline_from_autoai_on_wml(self):
        AbstractTestOBM.service_batch.create(
            experiment_run_id=self.remote_auto_pipelines._engine._current_run_id,
            model=self.pipeline_name_to_deploy,
            deployment_name=self.DEPLOYMENT_NAME + ' BATCH')

        self.assertIsNotNone(AbstractTestOBM.service_batch.id, msg="Batch Deployment creation - missing id")
        self.assertIsNotNone(AbstractTestOBM.service_batch.id, msg="Batch Deployment creation - name not set")
        self.assertIsNotNone(AbstractTestOBM.service_batch.asset_id,
                             msg="Batch Deployment creation - model (asset) id missing, incorrect model storing")

    def test_43_list_batch_deployments(self):
        deployments = AbstractTestOBM.service_batch.list()
        print(deployments)
        self.assertIn('created_at', str(deployments).lower())
        self.assertIn('status', str(deployments).lower())

        params = AbstractTestOBM.service_batch.get_params()
        print(params)
        self.assertIsNotNone(params)

    def test_45_run_job_batch_deployed_model_with_data_assets(self):
        data_connections_space_only = []

        test_case_batch_output_filename = "da_" + self.batch_output_filename

        results_reference = DataConnection(
            location=DeploymentOutputAssetLocation(name=test_case_batch_output_filename))

        if not self.BATCH_DEPLOYMENT_WITH_DA:
            self.skipTest("Skip batch deployment run job with data asset type")
        for node_name, data_filename in zip(self.input_node_names, self.input_data_filenames):
            asset_details = self.wml_client.data_assets.create(
                name=data_filename,
                file_path=self.input_data_path + data_filename)
            asset_id = asset_details['metadata']['guid']
            AbstractTestOBM.data_assets_to_delete.append(asset_id)

            conn_space = DataConnection(
                data_join_node_name=node_name,
                data_asset_id=asset_id)

            data_connections_space_only.append(conn_space)

        scoring_params = AbstractTestOBM.service_batch.run_job(
            payload=data_connections_space_only,
            output_data_reference=results_reference,
            background_mode=False)

        print(scoring_params)
        self.assertIsNotNone(scoring_params)

        deployment_job_id = self.wml_client.deployments.get_job_uid(scoring_params)

        self.wml_client.data_assets.list()

        data_asset_details = self.wml_client.data_assets.get_details()
        self.assertIn(test_case_batch_output_filename, str(data_asset_details),
                      f"Batch output file: {test_case_batch_output_filename} missed in data assets")

        predictions = AbstractTestOBM.service_batch.get_job_result(deployment_job_id)
        print(predictions)

        self.assertIsNotNone(predictions)

    def test_46_run_job_batch_deployed_model_with_data_assets_with_cos_connection(self):
        if not self.BATCH_DEPLOYMENT_WITH_CDA:
            self.skipTest("Skip batch deployment run job with data asset with cos connection type")

        data_connections_space_only = []
        # self.wml_client.set.default_space(self.target_space_id)

        test_case_batch_output_filename = "cos_da_" + self.batch_output_filename

        results_reference = DataConnection(
            location=DeploymentOutputAssetLocation(name=test_case_batch_output_filename))

        # prepare connection
        if not self.connection_id:
            connection_details = self.wml_client.connections.create({
                'datasource_type': self.wml_client.connections.get_datasource_type_uid_by_name(
                    'bluemixcloudobjectstorage'),
                'name': 'Connection to COS for tests',
                'properties': {
                    'bucket': self.bucket_name,
                    'access_key': self.cos_credentials['cos_hmac_keys']['access_key_id'],
                    'secret_key': self.cos_credentials['cos_hmac_keys']['secret_access_key'],
                    'iam_url': self.wml_client.service_instance._href_definitions.get_iam_token_url(),
                    'url': self.cos_endpoint
                }
            })

            self.connection_id = self.wml_client.connections.get_uid(connection_details)

        AbstractTestOBM.bucket_name = save_multiple_data_to_cos_bucket(self.input_data_path,
                                                                       data_filenames=self.input_data_filenames,
                                                                       data_cos_path=self.data_cos_path,
                                                                       access_key_id=
                                                                       self.cos_credentials['cos_hmac_keys'][
                                                                           'access_key_id'],
                                                                       secret_access_key=
                                                                       self.cos_credentials['cos_hmac_keys'][
                                                                           'secret_access_key'],
                                                                       cos_endpoint=self.cos_endpoint,
                                                                       bucket_name=self.bucket_name)

        # create connected data asset for each data set

        data_connections_space_only = []

        for node_name, data_filename in zip(self.input_node_names, self.input_data_filenames):
            asset_details = self.wml_client.data_assets.store({
                self.wml_client.data_assets.ConfigurationMetaNames.CONNECTION_ID: self.connection_id,
                self.wml_client.data_assets.ConfigurationMetaNames.NAME: f"Batch deployment asset - {node_name}",
                self.wml_client.data_assets.ConfigurationMetaNames.DATA_CONTENT_NAME: os.path.join(
                    self.data_cos_path, data_filename)
            })

            asset_id = self.wml_client.data_assets.get_id(asset_details)

            AbstractTestOBM.data_assets_to_delete.append(asset_id)

            self.assertIsInstance(asset_id, str)

            data_connections_space_only.append(
                DataConnection(data_join_node_name=node_name, data_asset_id=asset_id))

        scoring_params = AbstractTestOBM.service_batch.run_job(
            payload=data_connections_space_only,
            output_data_reference=results_reference,
            background_mode=False)

        print(scoring_params)
        self.assertIsNotNone(scoring_params)

        deployment_job_id = self.wml_client.deployments.get_job_uid(scoring_params)

        self.wml_client.data_assets.list()

        data_asset_details = self.wml_client.data_assets.get_details()
        self.assertIn(test_case_batch_output_filename, str(data_asset_details),
                      f"Batch output file: {test_case_batch_output_filename} missed in data assets")

        predictions = AbstractTestOBM.service_batch.get_job_result(deployment_job_id)
        print(predictions)

        self.assertIsNotNone(predictions)

    def test_47_run_job_batch_deployed_model_with_connected_data_asset(self):

        if not self.BATCH_DEPLOYMENT_WITH_CA:
            self.skipTest("Skip batch deployment run job with connected asset with cos connection type")
        data_connections_space_only = []
        self.wml_client.set.default_space(AbstractTestOBM.target_space_id)

        test_case_batch_output_filename = "cos_ca_" + self.batch_output_filename

        results_reference = DataConnection(
            location=DeploymentOutputAssetLocation(name=test_case_batch_output_filename))

        # prepare connection
        if not self.connection_id:
            connection_details = self.wml_client.connections.create({
                'datasource_type': self.wml_client.connections.get_datasource_type_uid_by_name(
                    'bluemixcloudobjectstorage'),
                'name': 'Connection to COS for tests',
                'properties': {
                    'bucket': self.bucket_name,
                    'access_key': self.cos_credentials['cos_hmac_keys']['access_key_id'],
                    'secret_key': self.cos_credentials['cos_hmac_keys']['secret_access_key'],
                    'iam_url': self.wml_client.service_instance._href_definitions.get_iam_token_url(),
                    'url': self.cos_endpoint
                }
            })

            self.connection_id = self.wml_client.connections.get_uid(connection_details)


        for node_name, data_filename in zip(self.input_node_names, self.input_data_filenames):
            conn_space = DataConnection(
                data_join_node_name=node_name,
                connection_asset_id=self.connection_id,
                location=S3Location(
                    bucket=self.bucket_name,
                    path=self.data_cos_path
                )
            )

            data_connections_space_only.append(conn_space)

        scoring_params = AbstractTestOBM.service_batch.run_job(
            payload=data_connections_space_only,
            output_data_reference=results_reference,
            background_mode=False)

        print(scoring_params)
        self.assertIsNotNone(scoring_params)

        deployment_job_id = self.wml_client.deployments.get_job_uid(scoring_params)

        self.wml_client.data_assets.list()

        data_asset_details = self.wml_client.data_assets.get_details()
        self.assertIn(test_case_batch_output_filename, str(data_asset_details),
                      f"Batch output file: {test_case_batch_output_filename} missed in data assets")

        predictions = AbstractTestOBM.service_batch.get_job_result(deployment_job_id)

        print(predictions)
        self.assertIsNotNone(predictions)

    def test_48_run_job_batch_deployed_model_with_data_connection_container(self):
        if self.wml_client.ICP or self.wml_client.WSD:
            self.skipTest("Batch Deployment with container data connection is available only for Cloud")
        else:
            self.skipTest("not ready")

    def test_49_delete_deployment_batch(self):
        print("Delete current deployment: {}".format(AbstractTestOBM.service_batch.deployment_id))
        AbstractTestOBM.service_batch.delete()
        self.wml_client.set.default_space(self.target_space_id) if not self.wml_client.default_space_id else None
        self.wml_client.repository.delete(AbstractTestOBM.service_batch.asset_id)

        print(f"Delete all created assets in Batch deployment tests. {AbstractTestOBM.data_assets_to_delete}")
        for asset_id in AbstractTestOBM.data_assets_to_delete:
            self.wml_client.data_assets.delete(asset_id)

        self.assertEqual(AbstractTestOBM.service_batch.id, None, msg="Deployment ID deleted incorrectly")
        self.assertEqual(AbstractTestOBM.service_batch.name, None, msg="Deployment name deleted incorrectly")
        self.assertEqual(AbstractTestOBM.service_batch.scoring_url, None,
                         msg="Deployment scoring_url deleted incorrectly")



import unittest
import logging
from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
from ibm_watson_machine_learning.tests.CP4D_35.preparation_and_cleaning import *


class TestWMLClientWithKeras(unittest.TestCase):
    deployment_uid = None
    model_uid = None
    scoring_url = None
    scoring_uid = None
    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestWMLClientWithKeras.logger.info("Service Instance: setting up credentials")

        self.wml_credentials = get_wml_credentials()
        self.client = get_client()





        self.space_name = str(uuid.uuid4())

        metadata = {
            self.client.spaces.ConfigurationMetaNames.NAME: 'client_space_' + self.space_name,
            self.client.spaces.ConfigurationMetaNames.DESCRIPTION: self.space_name + ' description'
        }

        self.space = self.client.spaces.store(meta_props=metadata)

        TestWMLClientWithKeras.space_id = self.client.spaces.get_id(self.space)
        print("space_id: ", TestWMLClientWithKeras.space_id)
        self.client.set.default_space(TestWMLClientWithKeras.space_id)

    def test_01_publish_local_model_in_repository(self):

        self.client.repository.ModelMetaNames.show()

        # model_content_path = 'artifacts/mnistCNN.h5.tgz'
        model_content_path = 'artifacts/tf_model_fvt_test.tar.gz'
        sw_spec_id = self.client.software_specifications.get_uid_by_name('default_py3.7')

        model_details = self.client.repository.store_model(model=model_content_path,
                                                           meta_props={self.client.repository.ModelMetaNames.NAME: "test_keras",
                                                                  self.client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_id,
                                                                  self.client.repository.ModelMetaNames.TYPE: 'tensorflow_2.1'})
        TestWMLClientWithKeras.model_id = self.client.repository.get_model_uid(model_details)

        print(TestWMLClientWithKeras.model_id)

        self.assertIsNotNone(TestWMLClientWithKeras.model_id)

    def test_02_get_details(self):
        TestWMLClientWithKeras.logger.info("Get details")
        self.assertIsNotNone(self.client.repository.get_details(TestWMLClientWithKeras.model_id))

    def test_03_create_deployment(self):
        TestWMLClientWithKeras.logger.info("Create deployment")
        deployment_details = self.client.deployments.create(TestWMLClientWithKeras.model_id,
                                                            meta_props={self.client.deployments.ConfigurationMetaNames.NAME: "Test keras deployment",
                                                                        self.client.deployments.ConfigurationMetaNames.ONLINE:{}})

        TestWMLClientWithKeras.logger.debug("Deployment details: {}".format(deployment_details))

        TestWMLClientWithKeras.deployment_id = self.client.deployments.get_id(deployment_details)

        TestWMLClientWithKeras.scoring_url = self.client.deployments.get_scoring_href(deployment_details)
        TestWMLClientWithKeras.logger.debug("Scoring url: {}".format(TestWMLClientWithKeras.scoring_url))

        self.assertTrue('online' in str(deployment_details))

    def test_04_get_deployment_details(self):
        TestWMLClientWithKeras.logger.info("Get deployment details")
        deployment_details = self.client.deployments.get_details(deployment_uid=TestWMLClientWithKeras.deployment_id)
        self.assertIsNotNone(deployment_details)

    def test_05_delete_model(self):
        TestWMLClientWithKeras.logger.info("Delete model")
        self.client.deployments.delete(TestWMLClientWithKeras.deployment_id)
        self.client.repository.delete(TestWMLClientWithKeras.model_id)

    def test_06_delete_space(self):
        TestWMLClientWithKeras.logger.info("Delete space")
        self.client.spaces.delete(TestWMLClientWithKeras.space_id)

if __name__ == '__main__':
    unittest.main()



"""
IBM Confidential
OCO Source Materials
5737-H76, 5725-W78, 5900-A1R
(c) Copyright IBM Corp. 2020 All Rights Reserved.
The source code for this program is not published or otherwise divested of its trade secrets,
irrespective of what has been deposited with the U.S. Copyright Office.
"""
import joblib
import numpy as np
import logging

from diffprivlib.models import GaussianNB

from ibmfl.exceptions import ModelInitializationException, ModelException
from ibmfl.model.model_update import ModelUpdate
from ibmfl.model.sklearn_fl_model import SklearnFLModel

logger = logging.getLogger(__name__)


class NaiveBayesFLModel(SklearnFLModel):
    """
    Wrapper class for diffprivlib.models.GaussianNB classifier, which itself
    is an implementation of sklearn.naive_bayes.GaussianNB with differential
    privacy.
    """

    def __init__(self, model_name, model_spec, model=None, **kwargs):
        """
        Create a `NaiveBayesFLModel` instance from a diffprivlib.models.GaussianNB
        model. If model is provided, it will use it; otherwise it will take
        the model_spec to create the model.

        :param model_name: A name specifying the type of model, e.g., \
        naive_bayes
        :type model_name: `str`
        :param model_spec: A dictionary contains model specification
        :type model_spec: `dict`
        :param model: Compiled GaussianNB model
        :type model: `diffprivlib.models.GaussianNB`
        :param kwargs: A dictionary contains other parameter settings on \
         to initialize a Naive Bayes model.
        :type kwargs: `dict`
        """
        if model:
            if not issubclass(type(model), GaussianNB):
                raise ValueError('Compiled GaussianNB needs to be provided'
                                 '(diffprivlib.models.GaussianNB). '
                                 'Type provided' + str(type(model)))

        super().__init__(model_name,
                         model_spec,
                         sklearn_model=model,
                         **kwargs)

        self.old_vals = {"theta": None, "sigma": None, "class_count": None}
        self.model_trained = False
        self.model_type = 'Sklearn-NaiveBayes'

    def fit_model(self, train_data, fit_params=None, **kwargs):
        """
        Fits current model with provided training data.

        :param train_data: Training data a tuple given in the form \
        (x_train, y_train).
        :type train_data: `np.ndarray`
        :param fit_params: Not used, present here for API consistency by \
        convention.
        :return: None
        """
        self.model.fit(train_data[0], train_data[1])
        self.model_trained = True

    def update_model(self, model_update, **kwargs):
        """
        Update GaussianNB model with provided model_update.

        :param model_update: `ModelUpdate` object that contains \
        the required information to update the GaussianNB model.
        :type model_update: `ModelUpdate`
        :return: None
        """
        if model_update.get("class_count") is None:
            return

        self.model.theta_ = model_update.get("theta")
        self.model.sigma_ = model_update.get("sigma")
        self.model.class_count_ = model_update.get("class_count")

        self._store_old_values(self.model.theta_, self.model.sigma_,
                               self.model.class_count_)

    def get_model_update(self):
        """
        Generates a `ModelUpdate` object that will be sent to other entities.

        :return: ModelUpdate
        :rtype: `ModelUpdate`
        """
        if (not self.model_trained or
                not hasattr(self.model, "class_count_") or
                np.all(self.old_vals['class_count'] ==
                       self.model.class_count_)):
            return ModelUpdate(theta=None, sigma=None, class_count=None)
        if self.old_vals['class_count'] is None:
            return ModelUpdate(theta=self.model.theta_,
                               sigma=self.model.sigma_,
                               class_count=self.model.class_count_)

        class_count_update = self.model.class_count_ - \
            self.old_vals['class_count']

        nm = self.model.class_count_[:, np.newaxis]
        n = self.old_vals['class_count'][:, np.newaxis]
        m = class_count_update[:, np.newaxis]

        theta_update = self.model.theta_ * nm - self.old_vals['theta'] * n
        theta_update = theta_update / m

        sigma_update = nm * self.model.sigma_ - n * self.old_vals['sigma'] - \
            (n * m) / nm * (self.old_vals['theta'] - theta_update) ** 2

        return ModelUpdate(theta=theta_update,
                           sigma=sigma_update,
                           class_count=class_count_update)

    def evaluate(self, test_dataset, **kwargs):
        """
        Evaluates the model given testing data.
        :param test_dataset: Testing data, a tuple given in the form \
        (x_test, test) or a datagenerator of of type `keras.utils.Sequence`, 
        `keras.preprocessing.image.ImageDataGenerator`
        :type test_dataset: `np.ndarray`

        :param kwargs: Dictionary of metrics available for the model
        :type kwargs: `dict`
        """

        if type(test_dataset) is tuple:
            x_test = test_dataset[0]
            y_test = test_dataset[1]

            return self.evaluate_model(x_test, y_test)

        else:
            raise ModelException("Invalid test dataset!")

    def evaluate_model(self, x, y, **kwargs):
        """
        Evaluates the model given test data x and the corresponding labels y.

        :param x: Samples with shape as expected by the model.
        :type x: `np.ndarray`
        :param y: Corresponding true labels to x
        :type y: `np.ndarray`
        :param kwargs: Dictionary of model-specific arguments for evaluating \
        models. For example, sample weights accepted by model.score.
        :return: Dictionary with all evaluation metrics provided by \
        specific implementation.
        :rtype: `dict`
        """
        acc = {'score': self.model.score(x, y, **kwargs)}
        return acc

    def _store_old_values(self, theta, sigma, class_count):
        """
        Store old values of training parameters to allow reconstruction of
        parameters for newly-trained examples when model update is sought.
        """
        self.old_vals = {"theta": theta,
                         "sigma": sigma,
                         "class_count": class_count}

    @staticmethod
    def load_model_from_spec(model_spec):
        """
        Loads model from provided model_spec, where model_spec is a `dict`
        that contains the following items: model_spec['model_definition']
        contains the model definition as type diffprivlib.GaussianNB.

        :param model_spec: Model specification contains \
        a complied sklearn model.
        :param model_spec: `dict`
        :return: model
        :rtype: `sklearn.cluster`
        """
        model = None
        try:
            if 'model_definition' in model_spec:
                path = model_spec['model_definition']
                with open(path, 'rb') as f:
                    model = joblib.load(f)

                if not issubclass(type(model), GaussianNB):
                    raise ValueError('Provided complied model in model_spec '
                                     'should be of type GaussianNB.'
                                     'Instead they are:' + str(type(model)))
        except Exception as ex:
            raise ModelInitializationException('Model specification was '
                                               'badly formed. ', str(ex))
        return model



import unittest
from sklearn.externals.joblib import load
from lale.helpers import import_from_sklearn_pipeline
from lale.lib.sklearn import RandomForestClassifier, KNeighborsClassifier
from lale.lib.xgboost import XGBClassifier
from sklearn.datasets import load_breast_cancer
from lale.lib.lale import HyperoptCV
from utils.assertions import is_lale_pipeline_type
from lale.json_operator import to_json


class TestLalePipeline(unittest.TestCase):
    """
    Test Case Description:

    1. Load pipeline model from file, prepared by autoai service in Watson Machine Learning.
    2. Convert loaded pipeline model to LALE Pipeline.
    3. Remove last Pipeline stage.
    4. Add three estimator choices as a last Pipeline stage.
    5. Use HyperOpt to find best Pipeline (best last step estimator)

    """
    train_data = None
    train_target = None
    pipeline_model_path = f"./artifacts/BreastCancer/model.pickle"
    pipeline_model = None
    lale_pipeline = None
    new_planned_pipeline = None

    @classmethod
    def setUpClass(cls):
        """Setups training_data, training_target."""
        breast_cancer_data = load_breast_cancer()
        cls.train_data = breast_cancer_data.data
        cls.train_target = breast_cancer_data.target

    def test_01__load_auto_ai_pipeline_model__read_model_from_disk__model_correctly_loaded(self):
        TestLalePipeline.pipeline_model = load(TestLalePipeline.pipeline_model_path)

        print(f"Loaded Pipeline Model Steps: {TestLalePipeline.pipeline_model.steps}")

    def test_02__import_from_sklearn_pipeline__convert_loaded_model_to_lale_pipeline__lale_pipeline_created(self):
        TestLalePipeline.lale_pipeline = import_from_sklearn_pipeline(TestLalePipeline.pipeline_model)

        is_lale_pipeline_type(pipeline=TestLalePipeline.lale_pipeline)

        print(f"Converted Lale Pipeline Steps: {to_json(TestLalePipeline.lale_pipeline)}")

    def test_03__remove_last_pipeline_step__deleting_step__last_step_deleted(self):
        number_of_pipeline_steps = len(TestLalePipeline.lale_pipeline._steps)
        TestLalePipeline.lale_pipeline._steps.remove(TestLalePipeline.lale_pipeline._steps[-1])

        self.assertEqual(number_of_pipeline_steps - 1, len(TestLalePipeline.lale_pipeline._steps),
                         msg="Last step of the pipeline was not correctly removed.")

    def test_04__freeze_trainable__add_estimator_choices_as_a_last_pipeline_step__new_pipeline_created(self):
        TestLalePipeline.new_planned_pipeline = TestLalePipeline.lale_pipeline.freeze_trainable() >> (
                RandomForestClassifier | KNeighborsClassifier | XGBClassifier
        )

        print(f"New planned Pipeline info: "
              f"{to_json(TestLalePipeline.new_planned_pipeline.steps()[-1])}")

        self.assertIn('RandomForestClassifier', str(to_json(TestLalePipeline.new_planned_pipeline.steps()[-1])),
                      msg="RandomForestClassifier is not in the pipeline last step.")
        self.assertIn('KNeighborsClassifier', str(to_json(TestLalePipeline.new_planned_pipeline.steps()[-1])),
                      msg="KNeighborsClassifier is not in the pipeline last step.")
        self.assertIn('XGBClassifier', str(to_json(TestLalePipeline.new_planned_pipeline.steps()[-1])),
                      msg="XGBClassifier is not in the pipeline last step.")

    def test_05__auto_configure__use_hyepopt_to_find_best_pipeline__best_pipeline_found(self):
        TestLalePipeline.new_best_pipeline = TestLalePipeline.new_planned_pipeline.auto_configure(
            X=TestLalePipeline.train_data, y=TestLalePipeline.train_target,
            optimizer=HyperoptCV, scoring='roc_auc', cv=3, max_evals=2
        )

        print(f"Best lale pipeline: {to_json(TestLalePipeline.new_best_pipeline)}")


if __name__ == '__main__':
    unittest.main()



#  (C) Copyright IBM Corp. 2020.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#       http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from pprint import pformat
from six import iteritems
import re

class MlAssetsModelSizeOutput(object):
    def __init__(self, contentSize=None):
        self.swagger_types = {
            'contentSize': 'str'
            }

        self.attribute_map = {
            'contentSize': 'contentSize'
        }

        self._contentSize = contentSize
    @property
    def contentSize(self):

        return self._contentSize

    @contentSize.setter
    def contentSize(self, contentSize):

        self._contentSize = contentSize

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other


import unittest

from ibm_watson_machine_learning.workspace import WorkSpace
from ibm_watson_machine_learning.deployment import Batch
from ibm_watson_machine_learning.helpers.connections import S3Connection, S3Location, DataConnection, AssetLocation
from tests.utils import get_wml_credentials, get_cos_credentials, is_cp4d
from sklearn.pipeline import Pipeline
import pandas as pd


class TestBatch(unittest.TestCase):
    pipeline: 'Pipeline' = None
    deployment: 'Batch' = None

    scoring_id: str = None
    # deployment_scikit: 'WebService' = None

    # deployment_id = '11fe2388-7cc8-481a-9db4-c2fb8b14739a'
    # deployment_name = 'autoAI deployment'
    # deployment_scoring_url = 'https://us-south.ml.cloud.ibm.com/v4/deployments/11fe2388-7cc8-481a-9db4-c2fb8b14739a/predictions'

    # #cpd svt17
    # space_id = 'd651b5f4-1fa0-40de-9e80-cebd5098b57a'
    # project_id = '94a6074d-48db-4279-bacb-90cd6f3358c7'

    # cpd qa41
    space_id = 'aa81ee79-795a-4a23-8265-e87966dfe437'
    project_id = '3d177338-3f8b-444d-b385-9ff929fd124f'
    autoai_run_id = '0987e342-2ef7-4cb0-aac2-108b81367db7'

    deployment_id = None

    deployment_name = "Unit-Bank_Batch-Deployment_v2"

    data_location = './autoai/data/bank.csv'

    bucket_name = "wml-autoai-tests"
    data_cos_path = 'data/bank.csv'

    prediction_column = 'y'

    data = None
    X = None

    @classmethod
    def setUp(cls) -> None:
        cls.data = pd.read_csv(cls.data_location)
        cls.X = cls.data.drop([cls.prediction_column], axis=1)
        cls.wml_credentials = get_wml_credentials()
        if not is_cp4d():
            cls.cos_credentials = get_cos_credentials()


    def test__01__initialize__Batch_initialization__object_initialized(self):
        TestBatch.deployment = Batch(self.wml_credentials.copy(),
                                          project_id=self.project_id,
                                          space_id=self.space_id)

        self.assertIsInstance(self.deployment, Batch, msg="Deployment is not of Batch type.")
        self.assertIsInstance(self.deployment._workspace, WorkSpace, msg="Workspace set incorrectly.")
        self.assertEqual(self.deployment.id, None, msg="Deployment ID initialized incorrectly")
        self.assertEqual(self.deployment.name, None, msg="Deployment name initialized incorrectly")

    def test__02__list__listing_deployments__deployments_listed_as_pandas_data_frame(self):
        deployments = self.deployment.list()
        print(deployments)

        self.assertIsInstance(deployments, pd.DataFrame, msg="Deployments are not of pandas DaraFrame type.")

    # def test__03__list__listing_deployments_with_limit__one_deployment_listed_as_pandas_data_frame(self):
    #     deployments = self.deployment.list(limit=1)
    #     print(deployments)
    #
    #     self.assertEqual(len(deployments), 1, msg="There is a different number of deployments than 1.")

    # def test__04__get__fetching_deployment__Batch_object_correctly_populated(self):
    #     self.deployment.get(deployment_id=self.deployment_id)
    #
    #     self.assertEqual(self.deployment.id, self.deployment_id, msg="Deployment ID initialized incorrectly")
    #     self.assertEqual(self.deployment.name, self.deployment_name, msg="Deployment name initialized incorrectly")
    #     # self.assertEqual(self.deployment.scoring_url, self.deployment_scoring_url,
    #     #                  msg="Deployment scoring_url initialized incorrectly")

    def test__05__get_params__fetching_deployment_details__details_fetched(self):
        details = self.deployment.get_params()
        print(details)

        self.assertIsInstance(details, dict, msg="Deployment parameters are not of dict type.")

    def test__06__create__creating_a_deployment__deployment_created(self):
        if not self.deployment_id:
            TestBatch.deployment.create(experiment_run_id=self.autoai_run_id,
                                        model='Pipeline_1',
                                        deployment_name=self.deployment_name)
            self.assertNotEqual(self.deployment.id, None, msg="Deployment ID initialized incorrectly")
            self.assertNotEqual(self.deployment.name, None, msg="Deployment name initialized incorrectly")
        else:
            print('Using existing deployment, id = ', self.deployment_id)
            TestBatch.deployment.get(deployment_id=self.deployment_id)

    # def test__07__score_DataFrame(self):
    #     payload_df = self.X[:5]
    #     scoring_results = self.deployment.score(payload_df, background_mode=False)
    #     print(scoring_results)



    def test__08__score_DataConnection(self):
        if self.deployment._workspace.wml_client.ICP_30:
            self.deployment._workspace.wml_client.set.default_space(self.space_id)
            metadata = {
                self.deployment._workspace.wml_client.data_assets.ConfigurationMetaNames.NAME: self.data_location.split('/')[-1],
                self.deployment._workspace.wml_client.data_assets.ConfigurationMetaNames.DATA_CONTENT_NAME: self.data_location
            }
            asset_details = self.deployment._workspace.wml_client.data_assets.store(meta_props=metadata)
            print(asset_details)
            payload_ref = [{"location": {"href": self.deployment._workspace.wml_client.data_assets.get_href(asset_details)},
                            "type": "data_asset",
                            "connection": {}
                            }]
        else:
            payload_ref = [DataConnection(
                connection=S3Connection(endpoint_url=self.cos_credentials['endpoint_url'],
                                        access_key_id=self.cos_credentials['access_key_id'],
                                        secret_access_key=self.cos_credentials['secret_access_key']),
                location=S3Location(bucket=self.bucket_name,
                                    path=self.data_cos_path))]

        out_ref = "unit_test_bank_out"

        scoring_results = self.deployment.score(payload=payload_ref, output_data_filename=out_ref)
        print(scoring_results)

        TestBatch.scoring_id = scoring_results['metadata']['id']

    def test__09__get_scoring_params(self):
        job_details = self.deployment.get_scoring_params(self.scoring_id)
        print(job_details)

        self.assertIsInstance(job_details, dict)

    def test__10__get_scoring_status(self):
        job_status = self.deployment.get_scoring_status(self.scoring_id)
        print(job_status)

        self.assertIsInstance(job_status, dict)

    def test__11__get_scoring_result(self):
        result = self.deployment.get_scoring_result(self.scoring_id)
        print(result)
        self.assertIsNone(result)

    def test__12__score_rerun(self):
        scoring_details = self.deployment.score_rerun(self.scoring_id, background_mode=True)

        self.assertIsInstance(scoring_details, dict)

        print(self.deployment.get_scoring_status(scoring_details['metadata']['id']))

    def test__13__list_scoring_jobs(self):
        scoring_jobs_df = self.deployment.list_scorings()
        print(scoring_jobs_df)
        self.assertGreater(len(scoring_jobs_df), 0)

    def test__14__delete__deleting_deployment__deployment_deleted(self):
        print(f"Deleting deployment: {self.deployment}")
        self.deployment.delete()

        self.assertEqual(self.deployment.id, None, msg="Deployment ID deleted incorrectly")
        self.assertEqual(self.deployment.name, None, msg="Deployment name deleted incorrectly")
        self.assertEqual(self.deployment.scoring_url, None,
                         msg="Deployment scoring_url deleted incorrectly")


if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .content_reader import ContentReader
from .ml_repository_api import MLRepositoryApi
from .ml_repository_client import MLRepositoryClient
from .model_adapter import ModelAdapter
from .model_collection import ModelCollection
from .experiment_adapter import ExperimentAdapter
from .experiment_collection import ExperimentCollection
from .ml_repository_client import connect
from .wml_experiment_collection import WmlExperimentCollection
from .wml_experiment_adapter import WmlExperimentCollectionAdapter
from .libraries_adapter import WmlLibrariesAdapter
from .libraries_collection import LibrariesCollection
from .runtimes_adapter import WmlRuntimesAdapter
from .runtimes_collection import RuntimesCollection


__all__ = ['ContentReader', 'MLRepositoryApi', 'MLRepositoryClient', 'ModelAdapter', 'ModelCollection',
           'ExperimentAdapter', 'ExperimentCollection', 'connect', 'WmlExperimentCollection', 'WmlExperimentCollectionAdapter', 'WmlLibrariesAdapter', 'LibrariesCollection',
           'RuntimesCollection', 'WmlRuntimesAdapter']



import unittest


from ibm_watson_machine_learning import APIClient

from ibm_watson_machine_learning.experiment import AutoAI

from ibm_watson_machine_learning.experiment.autoai.optimizers import RemoteAutoPipelines
from ibm_watson_machine_learning.tests.utils import get_wml_credentials, get_cos_credentials, is_cp4d
from ibm_watson_machine_learning.utils.autoai.errors import TShirtSizeNotSupported
from ibm_watson_machine_learning.utils.autoai.enums import TShirtSize


class TestAutoAIRemote(unittest.TestCase):
    wml_credentials = None

    DEV_FIELD_NAME = 'development'
    OPTIMIZER_NAME = 'Test t-shirt size'

    experiment: 'AutoAI' = None

    supported_tshirt_size = TShirtSize.L


    @classmethod
    def setUp(cls) -> None:
        """
        Load WML credentials from config.ini file based on ENV variable.
        """
        cls.wml_credentials = get_wml_credentials()

        cls.supported_tshirt_size = TShirtSize.L

    def test_01_wml_credentials_development_False(self):
        if self.DEV_FIELD_NAME in self.wml_credentials.keys():
            self.wml_credentials.pop(self.DEV_FIELD_NAME)
        self.assertNotIn(self.DEV_FIELD_NAME, self.wml_credentials.keys())

    def test_02_initialize_AutoAI_experiment(self):
        TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials)

        self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")

    def test_03_initialize_optimizer(self):
        TestAutoAIRemote.remote_auto_pipelines = self.experiment.optimizer(
            name=self.OPTIMIZER_NAME,
            prediction_type=AutoAI.PredictionType.MULTICLASS,
            prediction_column="column",
            scoring=AutoAI.Metrics.ACCURACY_SCORE,
            t_shirt_size=self.supported_tshirt_size
        )

        self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
                              msg="experiment.optimizer did not return RemoteAutoPipelines object")

    def test_04_initialize_optimizer_set_unsupported_tshirt_size(self):
        tshirt_size =TShirtSize.M if self.supported_tshirt_size == TShirtSize.L else TShirtSize.L
        print(f"Unsupported tshirt size tested: {tshirt_size}")
        with self.assertRaises(TShirtSizeNotSupported):
            _ = self.experiment.optimizer(
                name=self.OPTIMIZER_NAME,
                prediction_type=AutoAI.PredictionType.MULTICLASS,
                prediction_column="column",
                scoring=AutoAI.Metrics.ACCURACY_SCORE,
                t_shirt_size=tshirt_size
            )

        print("TShirtSizeNotSupported error raised succesfully")

    def test_05_initialize_optimizer_set_unsupported_tshirt_size(self):
        with self.assertRaises(TShirtSizeNotSupported):
            _ = self.experiment.optimizer(
                name=self.OPTIMIZER_NAME,
                prediction_type=AutoAI.PredictionType.MULTICLASS,
                prediction_column="column",
                scoring=AutoAI.Metrics.ACCURACY_SCORE,
                t_shirt_size=TShirtSize.XL
            )

        print("TShirtSizeNotSupported error raised succesfully")

    # development credentials #

    def test_06_wml_credentials_add_development_True(self):

        self.wml_credentials[self.DEV_FIELD_NAME] = True
        self.assertIn(self.DEV_FIELD_NAME, self.wml_credentials.keys())

        if is_cp4d():
            TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials,
                                                 project_id=self.project_id,
                                                 space_id=self.space_id)
        else:
            TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials)

        self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")

    def test_07_initialize_optimizer(self):
        for tshirt_size in [TShirtSize.S, TShirtSize.M, TShirtSize.L, TShirtSize.XL]:
            try:
                TestAutoAIRemote.remote_auto_pipelines = self.experiment.optimizer(
                    name=self.OPTIMIZER_NAME,
                    prediction_type=AutoAI.PredictionType.MULTICLASS,
                    prediction_column="column",
                    scoring=AutoAI.Metrics.ACCURACY_SCORE,
                    t_shirt_size=tshirt_size
                )

                self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
                                      msg="experiment.optimizer did not return RemoteAutoPipelines object")
            except TShirtSizeNotSupported as e:
                print("Initializing optimalizer fails with TShirtSizeNotSupported error.")
                self.assertTrue(False,
                                msg=f"Initializing optimalizer fails with TShirtSizeNotSupported error."
                                    f"\n tshirt_size = {tshirt_size}\n {e}")










import unittest
import logging
from sklearn import datasets
from sklearn.pipeline import Pipeline
from sklearn import preprocessing
from sklearn import svm
from ibm_watson_machine_learning.wml_client_error import WMLClientError
from ibm_watson_machine_learning.tests.Cloud.preparation_and_cleaning import *
from ibm_watson_machine_learning.tests.Cloud.models_preparation import *


class TestWMLClientWithScikitLearn(unittest.TestCase):
    deployment_id = None
    model_id = None
    scoring_url = None
    space_id = None
    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestWMLClientWithScikitLearn.logger.info("Service Instance: setting up credentials")

        self.wml_credentials = get_wml_credentials()
        self.client = get_client()
        self.model_data = create_scikit_learn_model_data()
        self.cos_credentials = get_cos_credentials()
        self.cos_resource_crn = self.cos_credentials['resource_instance_id']
        self.space_name = str(uuid.uuid4())

        self.instance_crn = get_instance_crn()

    def test_00_set_space(self):
        metadata = {
            self.client.spaces.ConfigurationMetaNames.NAME: 'space' + self.space_name,
            self.client.spaces.ConfigurationMetaNames.DESCRIPTION: self.space_name + ' description',
            self.client.spaces.ConfigurationMetaNames.STORAGE: {
                "type": "bmcos_object_storage",
                "resource_crn": self.cos_resource_crn
            },
            self.client.spaces.ConfigurationMetaNames.COMPUTE: {
                "name": "existing_instance_id",
                "crn": self.instance_crn
            }
        }

        self.space = self.client.spaces.store(meta_props=metadata, background_mode=False)

        print(self.space)

        self.client.spaces.list()

        TestWMLClientWithScikitLearn.space_id = self.client.spaces.get_id(self.space)
        print("space_id: ", TestWMLClientWithScikitLearn.space_id)

        self.client.set.default_space(TestWMLClientWithScikitLearn.space_id)
        # self.client.set.default_space('5fad8290-9c49-4403-b5f1-0cef4e061e00')
        self.assertTrue("SUCCESS" in self.client.set.default_space(TestWMLClientWithScikitLearn.space_id))

    def test_01_publish_model(self):
        TestWMLClientWithScikitLearn.logger.info("Creating scikit-learn model ...")

        predicted = self.model_data['prediction']

        TestWMLClientWithScikitLearn.logger.debug(predicted)
        self.assertIsNotNone(predicted)

        self.logger.info("Publishing scikit-learn model ...")

        self.client.repository.ModelMetaNames.show()

        sw_spec_id = self.client.software_specifications.get_uid_by_name("scikit-learn_0.20-py3.6")
        #sw_spec_uid = '09c5a1d0-9c1e-4473-a344-eb7b665ff687'
        print('sw_space_uid: ', sw_spec_id)
        # input_schema = input_data_schema = {
        #                                 "id": "test1",
        #                                 "type": "list",
        #                                 "fields": [{
        #                                     "name": "id",
        #                                     "type": "double",
        #                                     "nullable": True,
        #                                   }]
        #                                 }
        model_props = {self.client.repository.ModelMetaNames.NAME: "ScikitModel",
            self.client.repository.ModelMetaNames.TYPE: "scikit-learn_0.20",
            self.client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_id #,
            # self.client.repository.ModelMetaNames.INPUT_DATA_SCHEMA: input_schema
                       }
        published_model_details = self.client.repository.store_model(model=self.model_data['model'], meta_props=model_props, training_data=self.model_data['training_data'], training_target=self.model_data['training_target'])
        print(published_model_details)
        TestWMLClientWithScikitLearn.model_id = self.client.repository.get_model_id(published_model_details)
        TestWMLClientWithScikitLearn.model_url = self.client.repository.get_model_href(published_model_details)
        self.logger.info("Published model ID:" + str(TestWMLClientWithScikitLearn.model_id))
        self.logger.info("Published model URL:" + str(TestWMLClientWithScikitLearn.model_url))
        self.assertIsNotNone(TestWMLClientWithScikitLearn.model_id)
        self.assertIsNotNone(TestWMLClientWithScikitLearn.model_url)

    def test_02_download_model(self):
        TestWMLClientWithScikitLearn.logger.info("Download model")
        try:
            os.remove('download_test_url')
        except OSError:
            pass

        try:
            file = open('download_test_id', 'r')
        except IOError:
            file = open('download_test_id', 'w')
            file.close()

        self.client.repository.download(TestWMLClientWithScikitLearn.model_id, filename='download_test_url')
        self.assertRaises(WMLClientError, self.client.repository.download, TestWMLClientWithScikitLearn.model_id, filename='download_test_id')

    def test_03_get_details(self):
        TestWMLClientWithScikitLearn.logger.info("Get details")
        self.assertIsNotNone(self.client.repository.get_details())
        det = self.client.repository.get_details()
        print(det)

    def test_04a_get_model_details(self):
        TestWMLClientWithScikitLearn.logger.info("Get model details")
        details = self.client.repository.get_details(TestWMLClientWithScikitLearn.model_id)
        import json
        print(details)
        self.assertIsNotNone(self.client.repository.get_model_details(TestWMLClientWithScikitLearn.model_id))

    def test_04b_get_models_details_sync(self):
        TestWMLClientWithScikitLearn.logger.info("Get all models details sync")
        details = self.client.repository.get_model_details()
        print(details)

        details = self.client.repository.get_model_details(limit=10)
        print(details)

    def test_04c_get_models_details_async(self):
        TestWMLClientWithScikitLearn.logger.info("Get all models details async")
        for details in self.client.repository.get_model_details(limit=10, asynchronous=True,
                                                                get_all=True):
            print(details)

    def test_04d_list_models_sync(self):
        TestWMLClientWithScikitLearn.logger.info("List models sync")
        self.client.repository.list_models()
        self.client.repository.list_models(limit=10)

    def test_04e_list_models_async(self):
        TestWMLClientWithScikitLearn.logger.info("List models async")
        [details for details in self.client.repository.list_models(limit=10, asynchronous=True,
                                                                   get_all=True)]

    # def test_05_create_revision(self):
    #     TestWMLClientWithScikitLearn.logger.info("Create Revision")
    #     self.client.repository.create_model_revision(TestWMLClientWithScikitLearn.model_id)
    #     self.client.repository.list_models_revisions(TestWMLClientWithScikitLearn.model_id)
    #     model_meta_props = {self.client.repository.ModelMetaNames.NAME: "updated scikit model",
    #                         }
    #     published_model_updated = self.client.repository.update_model(TestWMLClientWithScikitLearn.model_id,
    #                                                                   model_meta_props, self.model_data['model'])
    #     self.assertIsNotNone(TestWMLClientWithScikitLearn.model_id)
    #     self.assertTrue("updated scikit model" in str(published_model_updated))
    #     self.client.repository.create_model_revision(TestWMLClientWithScikitLearn.model_id)
    #     rev_details = self.client.repository.get_model_revision_details(TestWMLClientWithScikitLearn.model_id, 2)
    #     self.assertTrue("updated scikit model" in str(rev_details))

    # def test_06_create_deployment(self):
    #     TestWMLClientWithScikitLearn.logger.info("Create deployments")
    #
    #     # # Temporary workaround
    #     # metadata = {
    #     #     self.client.spaces.ConfigurationMetaNames.NAME: "updated_space",
    #     #     self.client.spaces.ConfigurationMetaNames.COMPUTE: {"name": "test_instance",
    #     #                                                         "crn": self.instance_crn
    #     #                                                        }
    #     # }
    #     #
    #     # space_update_details = self.client.spaces.update(TestWMLClientWithScikitLearn.space_id, metadata)
    #     #
    #     # print("Updated space: ", space_update_details)
    #
    #     deployment = self.client.deployments.create(self.model_id, meta_props={
    #         self.client.deployments.ConfigurationMetaNames.NAME: "Test deployment",
    #         self.client.deployments.ConfigurationMetaNames.ONLINE: {}})
    #
    #     print('Deployment: ', deployment)
    #
    #     TestWMLClientWithScikitLearn.deployment_id = self.client.deployments.get_id(deployment)
    #     TestWMLClientWithScikitLearn.logger.info("model_id: " + self.model_id)
    #     TestWMLClientWithScikitLearn.logger.info("Online deployment: " + str(deployment))
    #     self.assertTrue(deployment is not None)
    #     TestWMLClientWithScikitLearn.scoring_url = self.client.deployments.get_scoring_href(deployment)
    #     self.assertTrue("online" in str(deployment))
    #     #self.client.deployments.get_status(TestWMLClientWithScikitLearn.deployment_id)
    #
    # def test_07_get_deployment_details(self):
    #     TestWMLClientWithScikitLearn.logger.info("Get deployment details")
    #     deployment_details = self.client.deployments.get_details()
    #     print(deployment_details)
    #     self.assertTrue("Test deployment" in str(deployment_details))
    #
    # def test_08_get_deployment_details_using_id(self):
    #     TestWMLClientWithScikitLearn.logger.info("Get deployment details using uid")
    #     deployment_details = self.client.deployments.get_details(TestWMLClientWithScikitLearn.deployment_id)
    #     print(deployment_details)
    #     self.assertIsNotNone(deployment_details)
    #
    # def test_09_score(self):
    #     TestWMLClientWithScikitLearn.logger.info("Score model")
    #     scoring_data = {
    #         self.client.deployments.ScoringMetaNames.INPUT_DATA: [
    #             {
    #                'values': [[0.0, 0.0, 5.0, 16.0, 16.0, 3.0, 0.0, 0.0, 0.0, 0.0, 9.0, 16.0, 7.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.0, 15.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 15.0, 16.0, 15.0, 4.0, 0.0, 0.0, 0.0, 0.0, 9.0, 13.0, 16.0, 9.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0, 12.0, 0.0, 0.0, 0.0, 0.0, 5.0, 12.0, 16.0, 8.0, 0.0, 0.0, 0.0, 0.0, 3.0, 15.0, 15.0, 1.0, 0.0, 0.0], [0.0, 0.0, 6.0, 16.0, 12.0, 1.0, 0.0, 0.0, 0.0, 0.0, 5.0, 16.0, 13.0, 10.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.0, 5.0, 15.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 8.0, 15.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0, 13.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.0, 16.0, 9.0, 4.0, 1.0, 0.0, 0.0, 3.0, 16.0, 16.0, 16.0, 16.0, 10.0, 0.0, 0.0, 5.0, 16.0, 11.0, 9.0, 6.0, 2.0]]
    #             }
    #         ]
    #     }
    #     predictions = self.client.deployments.score(TestWMLClientWithScikitLearn.deployment_id, scoring_data)
    #     print("predictions: ", predictions)
    #     self.assertTrue("prediction" in str(predictions))

    # def test_05_score(self):
    #     TestWMLClientWithScikitLearn.logger.info("Score model")
    #     scoring_data = {
    #         self.client.deployments.ScoringMetaNames.INPUT_DATA: [
    #             {
    #                'values': [[0.0, 0.0, 5.0, 16.0, 16.0, 3.0, 0.0, 0.0, 0.0, 0.0, 9.0, 16.0, 7.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.0, 15.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 15.0, 16.0, 15.0, 4.0, 0.0, 0.0, 0.0, 0.0, 9.0, 13.0, 16.0, 9.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0, 12.0, 0.0, 0.0, 0.0, 0.0, 5.0, 12.0, 16.0, 8.0, 0.0, 0.0, 0.0, 0.0, 3.0, 15.0, 15.0, 1.0, 0.0, 0.0], [0.0, 0.0, 6.0, 16.0, 12.0, 1.0, 0.0, 0.0, 0.0, 0.0, 5.0, 16.0, 13.0, 10.0, 0.0, 0.0, 0.0, 0.0, 0.0, 5.0, 5.0, 15.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 8.0, 15.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0, 13.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.0, 16.0, 9.0, 4.0, 1.0, 0.0, 0.0, 3.0, 16.0, 16.0, 16.0, 16.0, 10.0, 0.0, 0.0, 5.0, 16.0, 11.0, 9.0, 6.0, 2.0]]
    #             }
    #         ]
    #     }
    #     predictions = self.client.deployments.score(TestWMLClientWithScikitLearn.deployment_id, scoring_data)
    #     self.assertTrue("prediction" in str(predictions))


    # def test_10_delete_deployment(self):
    #     TestWMLClientWithScikitLearn.logger.info("Delete deployment")
    #     self.client.deployments.delete(TestWMLClientWithScikitLearn.deployment_id)

    def test_11_delete_model(self):
        TestWMLClientWithScikitLearn.logger.info("Delete model")
        self.client.repository.delete(TestWMLClientWithScikitLearn.model_id)

    def test_12_delete_space(self):
        self.client.spaces.delete(TestWMLClientWithScikitLearn.space_id)


if __name__ == '__main__':
    unittest.main()


# coding: utf-8

"""
Copyright 2016 SmartBear Software

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Ref: https://github.com/swagger-api/swagger-codegen
"""

#  (C) Copyright IBM Corp. 2020.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#       http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from pprint import pformat
from six import iteritems


class ArtifactVersionRepository(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self):
        """
        ArtifactVersionRepository - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'guid': 'UUID',
            'url': 'str',
            'content_url': 'str'
        }

        self.attribute_map = {
            'guid': 'guid',
            'url': 'url',
            'content_url': 'content_url'
        }

        self._guid = None
        self._url = None
        self._content_url = None

    @property
    def guid(self):
        """
        Gets the guid of this ArtifactVersionRepository.


        :return: The guid of this ArtifactVersionRepository.
        :rtype: UUID
        """
        return self._guid

    @guid.setter
    def guid(self, guid):
        """
        Sets the guid of this ArtifactVersionRepository.


        :param guid: The guid of this ArtifactVersionRepository.
        :type: UUID
        """
        self._guid = guid

    @property
    def url(self):
        """
        Gets the url of this ArtifactVersionRepository.


        :return: The url of this ArtifactVersionRepository.
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """
        Sets the url of this ArtifactVersionRepository.


        :param url: The url of this ArtifactVersionRepository.
        :type: str
        """
        self._url = url

    @property
    def content_url(self):
        """
        Gets the content_url of this ArtifactVersionRepository.


        :return: The content_url of this ArtifactVersionRepository.
        :rtype: str
        """
        return self._content_url

    @content_url.setter
    def content_url(self, content_url):
        """
        Sets the content_url of this ArtifactVersionRepository.


        :param content_url: The content_url of this ArtifactVersionRepository.
        :type: str
        """
        self._content_url = content_url

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other




import unittest
from sklearn.pipeline import Pipeline
from pprint import pprint
import pandas as pd
import traceback

from ibm_watson_machine_learning import APIClient

from ibm_watson_machine_learning.experiment import AutoAI
from ibm_watson_machine_learning.deployment import WebService

from ibm_watson_machine_learning.helpers.connections import S3Connection, S3Location, DataConnection, AssetLocation

from ibm_watson_machine_learning.experiment.autoai.optimizers import RemoteAutoPipelines

from ibm_watson_machine_learning.tests.utils import get_wml_credentials, get_cos_credentials, is_cp4d, get_env
from ibm_watson_machine_learning.tests.utils.cleanup import delete_model_deployment

from lale import wrap_imported_operators
from lale.operators import TrainablePipeline
from lale.lib.lale import Hyperopt


@unittest.skipIf(is_cp4d(), "Not supported on CP4D")
class TestAutoAIRemote(unittest.TestCase):
    wml_client: 'APIClient' = None
    experiment: 'AutoAI' = None
    remote_auto_pipelines: 'RemoteAutoPipelines' = None
    wml_credentials = None
    cos_credentials = None
    pipeline_opt: 'RemoteAutoPipelines' = None
    service: 'WebService' = None

    data_location = './autoai/data/bank.csv'

    trained_pipeline_details = None
    run_id = None

    data_connection = None
    results_connection = None
    sklearn_pipeline = None

    train_data = None
    holdout_data = None

    cos_endpoint = None
    bucket_name = None
    data_cos_path = 'data/bank.csv'

    project_id = None
    space_id = None

    results_cos_path = None

    pipeline: 'Pipeline' = None
    lale_pipeline = None
    deployed_pipeline = None
    hyperopt_pipelines = None
    new_pipeline = None
    new_sklearn_pipeline = None

    OPTIMIZER_NAME = 'Bank wml_autoai binary test'
    DEPLOYMENT_NAME = "Bank AutoAI Deployment tests"

    asset_id = None

    @classmethod
    def setUp(cls) -> None:
        """
        Load WML credentials from config.ini file based on ENV variable.
        """
        cls.data = pd.read_csv(cls.data_location)
        cls.X = cls.data.drop(['y'], axis=1)
        cls.y = cls.data['y']

        cls.wml_credentials = get_wml_credentials()
        if not is_cp4d():
            cls.cos_credentials = get_cos_credentials()
            cls.cos_endpoint = cls.cos_credentials['endpoint_url']
            cls.bucket_name = cls.cos_credentials['bucket_name']
            cls.results_cos_path = cls.cos_credentials['results_cos_path']

        cls.wml_client = APIClient(wml_credentials=cls.wml_credentials.copy())

    def test_01_initialize_AutoAI_experiment__pass_credentials__object_initialized(self):
        if is_cp4d():
            TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials.copy(),
                                                 project_id=self.project_id,
                                                 space_id=self.space_id)
        else:
            TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials)

        self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")

    def test_02_save_remote_data_and_DataConnection_setup(self):
        if is_cp4d():
            self.wml_client.set.default_project(self.project_id)
            asset_details = self.wml_client.data_assets.create(
                name=self.data_location.split('/')[-1],
                file_path=self.data_location)
            asset_id = asset_details['metadata']['guid']

            TestAutoAIRemote.data_connection = DataConnection(
                location=AssetLocation(asset_id=asset_id))
            TestAutoAIRemote.results_connection = None

        else:  # for cloud and COS
            TestAutoAIRemote.data_connection = DataConnection(
                connection=S3Connection(endpoint_url=self.cos_endpoint,
                                        access_key_id=self.cos_credentials['access_key_id'],
                                        secret_access_key=self.cos_credentials['secret_access_key']),
                location=S3Location(bucket=self.bucket_name,
                                    path=self.data_cos_path)
            )
            TestAutoAIRemote.results_connection = DataConnection(
                connection=S3Connection(endpoint_url=self.cos_endpoint,
                                        access_key_id=self.cos_credentials['access_key_id'],
                                        secret_access_key=self.cos_credentials['secret_access_key']),
                location=S3Location(bucket=self.bucket_name,
                                    path=self.results_cos_path)
            )
            TestAutoAIRemote.data_connection.write(data=self.data, remote_name=self.data_cos_path)

        self.assertIsNotNone(obj=TestAutoAIRemote.data_connection)

    def __make_experiment(self, name, scoring, test_size, max_est, daub_est):
        print('###############################################################')
        print(f"Preparing experiment: {name}")
        print('###############################################################')
        optimizer = self.experiment.optimizer(
            name=name,
            desc='',
            prediction_type=AutoAI.PredictionType.BINARY,
            prediction_column='y',
            scoring=scoring,
            test_size=test_size,
            max_number_of_estimators=max_est,
            daub_include_only_estimators=daub_est
        )

        print('###############################################################')
        print('Fetching experiment parameters...')
        print('###############################################################')
        parameters = optimizer.get_params()
        print(f'Optimizer parameters: {parameters}')
        print(parameters)

        print('###############################################################')
        print('Starting training...')
        print('###############################################################')
        optimizer.fit(
            training_data_reference=[self.data_connection],
            training_results_reference=self.results_connection,
            background_mode=False)

        print('###############################################################')
        print('Fetching training data...')
        print('###############################################################')
        TestAutoAIRemote.train_data, TestAutoAIRemote.holdout_data = optimizer.get_data_connections()[
            0].read(with_holdout_split=True)

        print("train data sample:")
        print(self.train_data.head())

        print("holdout data sample:")
        print(self.holdout_data.head())

        print('###############################################################')
        print('Fetching training status...')
        print('###############################################################')
        status = optimizer.get_run_status()
        print(f'Training status: {status}')

        print('###############################################################')
        print('Fetching training details...')
        print('###############################################################')
        parameters = optimizer.get_run_details()
        print(f'Training run details: {parameters}')

        print('###############################################################')
        print('Make prediction on the best pipeline...')
        print('###############################################################')
        predictions = optimizer.predict(X=self.holdout_data.drop(['y'], axis=1).values[:5])
        print(f'Optimizer predict: {predictions}')

        print('###############################################################')
        print('Fetching summary...')
        print('###############################################################')
        summary = optimizer.summary()
        print(f'Summary: {summary}')

        print('###############################################################')
        print('Fetching best pipeline details...')
        print('###############################################################')
        best_pipeline_params = optimizer.get_pipeline_details()
        print(f'Best pipeline parameters: {best_pipeline_params}')

        print('###############################################################')
        print('Starting exploring each computed pipeline...')
        print('###############################################################')
        for pipeline_name in summary.index.tolist():
            print('###############################################################')
            print(f'Fetching {pipeline_name} details...')
            print('###############################################################')
            pipeline_params = optimizer.get_pipeline_details(pipeline_name=pipeline_name)
            print(f'{pipeline_name} parameters: {pipeline_params}')

            print('###############################################################')
            print(f'Fetching {pipeline_name} as LALE...')
            print('###############################################################')
            lale_pipeline = optimizer.get_pipeline(pipeline_name=pipeline_name)
            print(f"Fetched pipeline type: {type(lale_pipeline)}")

            print('###############################################################')
            print('Make predictions...')
            print('###############################################################')
            predictions = lale_pipeline.predict(X=self.X.values[:10])
            print(f'Lale {pipeline_name} predicitons: {predictions}')

            print('###############################################################')
            print('Checking lale pretty print...')
            print('###############################################################')
            pipeline_code = lale_pipeline.pretty_print(combinators=False)
            try:
                exec(pipeline_code)
            except Exception as e:
                print('!!!!!!!!!!!!!!!!!!!!!!!EXCEPTION!!!!!!!!!!!!!!!!!!!!!!!!!!')
                traceback.print_exc()


            from lale.lib.sklearn import KNeighborsClassifier, DecisionTreeClassifier, LogisticRegression
            print('###############################################################')
            print('LALE new pipeline preparation...')
            print('###############################################################')
            prefix = lale_pipeline.remove_last().freeze_trainable()
            TestAutoAIRemote.new_pipeline = prefix >> (
                    KNeighborsClassifier | DecisionTreeClassifier | LogisticRegression)

            hyperopt = Hyperopt(estimator=TestAutoAIRemote.new_pipeline, cv=3, max_evals=20)
            try:
                print('###############################################################')
                print('Hyperopt fit...')
                print('###############################################################')
                hyperopt_pipelines = hyperopt.fit(self.X.values, self.y.values)
                new_pipeline_model = hyperopt_pipelines.get_pipeline()
                new_sklearn_pipeline = new_pipeline_model.export_to_sklearn_pipeline()
                predictions = new_sklearn_pipeline.predict(X=self.X.values[:20])
                print(f'sklearn hyperopt pipeline predictions: {predictions}')
            except Exception as e:
                traceback.print_exc()
                hyperopt_results = hyperopt._impl._trials.results
                print(f'Hyperopt results: {hyperopt_results}')

            print('###############################################################')
            print(f'Getting {pipeline_name} as SKLEARN... and store locally')
            print('###############################################################')
            TestAutoAIRemote.sklearn_pipeline = optimizer.get_pipeline(pipeline_name=pipeline_name,
                                                                       astype=self.experiment.PipelineTypes.SKLEARN,
                                                                       persist=True)
            print(f"Fetched pipeline type: {type(TestAutoAIRemote.sklearn_pipeline)}")

            print('###############################################################')
            print('Make predictions...')
            print('###############################################################')
            predictions = TestAutoAIRemote.sklearn_pipeline.predict(X=self.X.values[:10])
            print(f'Sklearn {pipeline_name} predicitons: {predictions}')

    def test_03__explore_experiments(self):

        cases = dict(
            names=[
                'bank_binary_RF',
                'bank_binary_XGB',
                'bank_binary_EX_TREES',
                'bank_binary_LGBM',
                'bank_binary_DT',
                'bank_binary_GB',
                'bank_binary_LR',
                'bank_binary_ALL_1',
                'bank_binary_ALL_2',
                'bank_binary_ALL_3',
                'bank_binary_ALL_4',

            ],
            scorings=[
                AutoAI.Metrics.ROOT_MEAN_SQUARED_ERROR,
                AutoAI.Metrics.ROC_AUC_SCORE,
                AutoAI.Metrics.LOG_LOSS,
                AutoAI.Metrics.F1_SCORE_WEIGHTED,
                AutoAI.Metrics.ACCURACY_SCORE,
                AutoAI.Metrics.RECALL_SCORE_MACRO,
                AutoAI.Metrics.R2_SCORE,
                AutoAI.Metrics.MEAN_SQUARED_LOG_ERROR,
                AutoAI.Metrics.EXPLAINED_VARIANCE_SCORE,
                AutoAI.Metrics.RECALL_SCORE,
                AutoAI.Metrics.ROOT_MEAN_SQUARED_LOG_ERROR,
            ],
            test_sizes=[
                0.1,
                0.2,
                0.15,
                0.05,
                0.18,
                0.09,
                0.12,
                0.07,
                0.1,
                0.1,
                0.1,
            ],
            max_estimators=[
                1,
                1,
                1,
                1,
                1,
                1,
                1,
                1,
                2,
                3,
                4,
            ],
            daub_estimators=[
                [AutoAI.ClassificationAlgorithms.RF],
                [AutoAI.ClassificationAlgorithms.XGB],
                [AutoAI.ClassificationAlgorithms.EX_TREES],
                [AutoAI.ClassificationAlgorithms.LGBM],
                [AutoAI.ClassificationAlgorithms.DT],
                [AutoAI.ClassificationAlgorithms.GB],
                [AutoAI.ClassificationAlgorithms.LR],
                None,
                None,
                None,
                None,
            ]
        )

        for name, scoring, test_size, max_est, daub_est in zip(cases['names'],
                                                               cases['scorings'],
                                                               cases['test_sizes'],
                                                               cases['max_estimators'],
                                                               cases['daub_estimators'],
                                                               ):
            try:
                self.__make_experiment(name, scoring, test_size, max_est, daub_est)

            except Exception as e:
                print(f'@@@@@@@@@@@@@@@@@@@@ EXPERIMENT ERROR @@@@@@@@@@@@@@@@@@@@@@@@@\n'
                      f'NAME: {name}\n'
                      f'{e}\n'
                      f'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n')
                traceback.print_exc()

    #################################
    #      DEPLOYMENT SECTION       #
    #################################

    def test_04_deployment_setup_and_preparation(self):
        if is_cp4d():
            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials.copy(),
                                                  source_project_id=self.project_id,
                                                  source_space_id=self.space_id)
        else:
            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials)

        delete_model_deployment(self.wml_client, deployment_name=self.DEPLOYMENT_NAME)

    def test_05__deploy__deploy_best_computed_pipeline_from_autoai_on_wml(self):
        self.service.create(
            experiment_run_id=self.remote_auto_pipelines._engine._current_run_id,
            model=self.pipeline,
            deployment_name=self.DEPLOYMENT_NAME)

    def test_06_score_deployed_model(self):
        nb_records = 10
        predictions = self.service.score(payload=self.holdout_data.drop(['y'], axis=1)[:nb_records])
        print(predictions)
        self.assertIsNotNone(predictions)
        self.assertEqual(len(predictions['predictions'][0]['values']), nb_records)

    def test_07_list_deployments(self):
        self.service.list()
        params = self.service.get_params()
        print(params)
        self.assertIsNotNone(params)

    def test_08_delete_deployment(self):
        print("Delete current deployment: {}".format(self.service.deployment_id))
        self.service.delete()


if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import requests
from ibm_watson_machine_learning.utils import STR_TYPE, STR_TYPE_NAME, docstring_parameter, meta_props_str_conv
from ibm_watson_machine_learning.metanames import RemoteTrainingSystemMetaNames
import os
import json
from ibm_watson_machine_learning.wml_client_error import WMLClientError, UnexpectedType, ApiRequestFailure
from ibm_watson_machine_learning.wml_resource import WMLResource
_DEFAULT_LIST_LENGTH = 50

class RemoteTrainingSystem(WMLResource):
    def __init__(self, client):
        WMLResource.__init__(self, __name__, client)

        self._client = client
        self.ConfigurationMetaNames = RemoteTrainingSystemMetaNames()

    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def store(self, meta_props):
        """
            Create a remote training system. Either space_id or project_id has to be provided and is mandatory.

           **Parameters**

           .. important::
                #. **meta_props**:  meta data. To see available meta names use **client.remote_training_systems.ConfigurationMetaNames.get()**\n
                   **type**: str or dict\n

           **Output**

           .. important::
                **returns**: Response json\n
                **return type**: dict\n

           **Example**

                >>> metadata = {
                >>>    client.remote_training_systems.ConfigurationMetaNames.NAME: "my-resource",
                >>>    client.remote_training_systems.ConfigurationMetaNames.TAGS: ["tag1", "tag2"],
                >>>    client.remote_training_systems.ConfigurationMetaNames.ORGANIZATION: {"name": "name", "region": "EU"}
                >>>    client.remote_training_systems.ConfigurationMetaNames.ALLOWED_IDENTITIES: [{"id": "43689024", "type": "user"}],
                >>>    client.remote_training_systems.ConfigurationMetaNames.REMOTE_ADMIN: {"name": "name", "email": "email@email.com"}
                >>> }
                >>> client.set.default_space('3fc54cf1-252f-424b-b52d-5cdd9814987f')
                >>> details = client.remote_training_systems.store(meta_props=metadata)
            """

        WMLResource._chk_and_block_create_update_for_python36(self)
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")
        
        self._client._check_if_either_is_set()

        RemoteTrainingSystem._validate_type(meta_props, u'meta_props', dict, True)
        self._validate_input(meta_props)

        meta = self.ConfigurationMetaNames._generate_resource_metadata(
            meta_props,
            with_validation=True,
            client=self._client)

        if self._client.default_space_id is not None:
            meta['space_id'] = self._client.default_space_id
        elif self._client.default_project_id is not None:
            meta['project_id'] = self._client.default_project_id

        href = self._client.service_instance._href_definitions.remote_training_systems_href()

        if not self._ICP:
            creation_response = requests.post(href,
                                              params=self._client._params(),
                                              headers=self._client._get_headers(),
                                              json=meta)
        else:
            creation_response = requests.post(href,
                                              params=self._client._params(),
                                              headers=self._client._get_headers(),
                                              json=meta,
                                              verify=False)

        details = self._handle_response(expected_status_code=201,
                                        operationName=u'store remote training system specification',
                                        response=creation_response)

        return details

    def _validate_input(self, meta_props):
        if 'name' not in meta_props:
            raise WMLClientError("Its mandatory to provide 'NAME' in meta_props. Example: "
                                 "client.remote_training_systems.ConfigurationMetaNames.NAME")

        if 'allowed_identities' not in meta_props:
            raise WMLClientError("Its mandatory to provide 'ALLOWED_IDENTITIES' in meta_props. Example: "
                                 "client.remote_training_systems.ConfigurationMetaNames.ALLOWED_IDENTITIES")

        if 'organization' in meta_props and 'name' not in meta_props[u'organization']:
            raise WMLClientError("Its mandatory to provide 'name' for ORGANIZATION meta_prop. Eg: "
                                 "client.remote_training_systems.ConfigurationMetaNames.ORGANIZATION: "
                                 "{'name': 'org'} ")


    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def delete(self, remote_training_systems_id):
        """
            Deletes the given remote_training_systems_id definition. 'space_id' or 'project_id' has to be provided

            **Parameters**

            .. important::
                #. **remote_training_systems_id**:  Remote Training System identifier\n
                   **type**: str\n

            **Output**

            .. important::
                **returns**: status ("SUCCESS" or "FAILED")\n
                **return type**: str\n

            **Example**

             >>> client.remote_training_systems.delete(remote_training_systems_id='6213cf1-252f-424b-b52d-5cdd9814956c')
        """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")

        self._client._check_if_either_is_set()
        RemoteTrainingSystem._validate_type(remote_training_systems_id, u'remote_training_systems_id', STR_TYPE, True)

        href = self._client.service_instance._href_definitions.remote_training_system_href(remote_training_systems_id)

        if not self._ICP:
            delete_response = requests.delete(href,
                                              params=self._client._params(),
                                              headers=self._client._get_headers())
        else:
            delete_response = requests.delete(href,
                                              params=self._client._params(),
                                              headers=self._client._get_headers(),
                                              verify=False)

        details = self._handle_response(expected_status_code=204,
                                        operationName=u'delete remote training system definition',
                                        response=delete_response)

        if "SUCCESS" == details:
            print("Remote training system deleted")

    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def get_details(self, remote_training_system_id=None, limit=None, asynchronous=False, get_all=False):
        """
            Get metadata of the given remote training system. if no remote_training_systems_id is specified all 
            remote training systems metadata is returned.

           **Parameters**

           .. important::
                #. **remote_training_system_id**: remote training system identifier (optional)\n
                   **type**: str\n
                #. **limit**:  limit number of fetched records (optional)\n
                   **type**: int\n
                #. **asynchronous**:  if True, it will work as a generator (optional)\n
                   **type**: bool\n
                #. **get_all**:  if True, it will get all entries in 'limited' chunks (optional)\n
                   **type**: bool\n

           **Output**

           .. important::
                **returns**: remote training system(s) metadata\n
                **return type**: dict\n
                dict (if remote_training_systems_id is not None) or {"resources": [dict]} (if remote_training_systems_id is None)\n

           .. note::
                If remote_training_systems_id is not specified, all remote training system(s) metadata is fetched\n

           **Example**

             >>> details = client.remote_training_systems.get_details(remote_training_systems_id)
             >>> details = client.remote_training_systems.get_details()
             >>> details = client.remote_training_systems.get_details(limit=100)
             >>> details = client.remote_training_systems.get_details(limit=100, get_all=True)
             >>> details = []
             >>> for entry in client.remote_training_systems.get_details(limit=100, asynchronous=True, get_all=True):
             >>>    details.extend(entry)
         """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")
        
        self._client._check_if_either_is_set()
        
        RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_systems_id', STR_TYPE, False)
        RemoteTrainingSystem._validate_type(limit, u'limit', int, False)

        href = self._client.service_instance._href_definitions.remote_training_systems_href()

        if remote_training_system_id is None:
            return self._get_artifact_details(href, remote_training_system_id, limit, 'remote_training_systems',
                                              _async=asynchronous, _all=get_all)
        else:
            return self._get_artifact_details(href, remote_training_system_id, limit, 'remote_training_systems')
    
    def list(self, limit=None):
        """
            List stored remote training systems. If limit is set to None there will be only first 50 records shown.

           **Parameters**

           .. important::
                #. **limit**:  limit number of fetched records\n
                   **type**: int\n

           **Output**

           .. important::
                This method only prints the list of all remote training systems in a table format.\n
                **return type**: None\n

           **Example**

            >>> client.remote_training_systems.list()
        """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")

        self._client._check_if_either_is_set()

        resources = self.get_details()[u'resources']

        values = [(m[u'metadata'][u'id'],
                   m[u'metadata'][u'name'],
                   m[u'metadata'][u'created_at']) for m in resources]

        self._list(values, [u'ID', u'NAME', u'CREATED'], limit, _DEFAULT_LIST_LENGTH)

    @staticmethod
    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def get_id(remote_training_system_details):
        """
                Get ID of remote_training_system

                **Parameters**

                .. important::

                   #. **remote_training_system_details**:  Metadata of the stored remote training system\n
                      **type**: dict\n

                **Output**

                .. important::

                    **returns**: ID of stored remote training system\n
                    **return type**: str\n

                **Example**

                 >>> details = client.remote_training_systems.get_details(remote_training_system_id)
                 >>> id = client.remote_training_systems.get_id(details)
        """
        RemoteTrainingSystem._validate_type(remote_training_system_details, u'remote_training_system_details', object, True)

        return WMLResource._get_required_element_from_dict(remote_training_system_details,
                                                           u'remote_training_system_details',
                                                           [u'metadata', u'id'])

    def update(self, remote_training_system_id, changes):
        """
        Updates existing remote training system metadata.

        **Parameters**

        .. important::
           #. **remote_training_system_id**:  remote training system identifier\n
              **type**: str\n
           #. **changes**:  elements which should be changed, where keys are ConfigurationMetaNames\n
              **type**: dict\n

        **Example**
         >>> metadata = {
         >>> client.remote_training_systems.ConfigurationMetaNames.NAME:"updated_remote_training_system"
         >>> }
         >>> details = client.remote_training_systems.update(remote_training_system_id, changes=metadata)

        """
        WMLResource._chk_and_block_create_update_for_python36(self)
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")

        self._client._check_if_either_is_set()

        self._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, True)
        self._validate_type(changes, u'changes', dict, True)
        meta_props_str_conv(changes)

        details = self.get_details(remote_training_system_id)

        patch_payload = self.ConfigurationMetaNames._generate_patch_payload(details['entity'], changes,
                                                                            with_validation=True)

        href = self._client.service_instance._href_definitions.remote_training_system_href(remote_training_system_id)

        if not self._ICP:
            response = requests.patch(href,
                                      json=patch_payload,
                                      params = self._client._params(),
                                      headers=self._client._get_headers())
        else:
            response = requests.patch(href,
                                      json=patch_payload,
                                      params = self._client._params(),
                                      headers=self._client._get_headers(),
                                      verify=False)

        updated_details = self._handle_response(200, u'remote training system patch', response)

        return updated_details

    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def create_revision(self, remote_training_system_id):
        """
            Create a new remote training system revision.

            :param remote_training_system_id: Unique remote training system ID
            :type remote_training_system_id: {str_type}

            Example:

            >>> client.remote_training_systems.create_revision(remote_training_system_id)
        """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")

        RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, False)

        href = self._client.service_instance._href_definitions.remote_training_systems_href()
        return self._create_revision_artifact(href, remote_training_system_id, 'remote training system')

    def get_revision_details(self, remote_training_system_id, rev_id):
        """
           Get metadata of specific revision of stored remote system

           :param remote_training_system_id: stored functions, definition
           :type remote_training_system_id: {str_type}

           :param rev_id: Unique id of the remote system revision.
           :type rev_id : int

           :returns: stored remote system revision metadata
           :rtype: dict

           Example:

           >>> details = client.remote_training_systems.get_details(remote_training_system_id, rev_id)
        """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")

        self._client._check_if_either_is_set()

        RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, True)
        RemoteTrainingSystem._validate_type(rev_id, u'rev_id', int, True)

        href = self._client.service_instance._href_definitions.remote_training_system_href(remote_training_system_id)
        return self._get_with_or_without_limit(href, limit=None, op_name="remote_training_system_id",
                                               summary=None, pre_defined=None, revision=rev_id)

    @docstring_parameter({'str_type': STR_TYPE_NAME})
    def list_revisions(self, remote_training_system_id, limit=None):
        """
           List all revisions for the given remote_training_system_id

           :param remote_training_system_id: Unique id of stored remote system
           :type remote_training_system_id: {str_type}

           :param limit: limit number of fetched records (optional)
           :type limit: int

           :returns: list all remote systems revisions summary.
           :rtype: table

           >>> details = client.remote_training_systems.list_revisions(remote_training_system_id)
        """
        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
            raise WMLClientError("Not supported in this release")
        
        self._client._check_if_either_is_set()

        RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, True)

        href = self._client.service_instance._href_definitions.get_function_href(remote_training_system_id)

        resources = self._get_artifact_details(href + '/revisions',
                                               None,
                                               limit,
                                               'remote system revisions')[u'resources']

        values = [(m[u'metadata'][u'id'],
             m[u'metadata'][u'rev'],
             m[u'metadata'][u'name'],
             m[u'metadata'][u'created_at']) for m in resources]

        self._list(values, [u'ID', u'rev', u'NAME', u'CREATED'], limit, _DEFAULT_LIST_LENGTH)



import os, random, struct
from Crypto.Cipher import AES
import sys

def encrypt_file(key, in_filename, out_filename=None, chunksize=64 * 1024):
    """ Encrypts a file using AES (CBC mode) with the
        given key.

        key:
            The encryption key - a string that must be
            either 16, 24 or 32 bytes long. Longer keys
            are more secure.

        in_filename:
            Name of the input file

        out_filename:
            If None, '<in_filename>.enc' will be used.

        chunksize:
            Sets the size of the chunk which the function
            uses to read and encrypt the file. Larger chunk
            sizes can be faster for some files and machines.
            chunksize must be divisible by 16.
    """
    if not out_filename:
        out_filename = in_filename + '.enc'
    iv = os.urandom(16)

    encryptor = AES.new(key, AES.MODE_CBC, iv)
    filesize = os.path.getsize(in_filename)

    with open(in_filename, 'rb') as infile:
        with open(out_filename, 'wb') as outfile:
            outfile.write(struct.pack('<Q', filesize))
            outfile.write(iv)

            while True:
                chunk = infile.read(chunksize)
                n = len(chunk)
                if len(chunk) == 0:
                    break
                elif len(chunk) % 16 != 0:
                    chunk += bytes(' ', 'utf-8') * (16 - n % 16)

                outfile.write(encryptor.encrypt(chunk))


try:
    key = sys.argv[1]
except:
    key = None
    print("Usage : ConfigFileEncoder.py  <16 bit key> <input config file> <output file>")
    sys.exit(1)

try:
    input_filename = sys.argv[2]
except:
    input_filename = None
    print("Usage : ConfigFileEncoder.py  <16 bit key> <input config file> <output file>")
    sys.exit(1)

try:
    output_filename = sys.argv[3]
except:
    output_filename = input_filename + '.enc'

encrypt_file(key, input_filename, output_filename)
